﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DataLayer
{
    public static class DataBufferExt
    {
        /// <summary>
        /// Gets 1 byte with offset check
        /// </summary>
        /// <param name="buffer"></param>
        /// <param name="offset"></param>
        /// <returns></returns>
        public static byte Get1Byte(this byte[] buffer, int offset)
        {
            return offset < buffer.Length ?
                    buffer[offset] :
                    (byte)0;
        }

        /// <summary>
        /// Gets 2 bytes with offset check
        /// </summary>
        /// <param name="buffer"></param>
        /// <param name="offset"></param>
        /// <returns></returns>
        public static UInt16 Get2Bytes(this byte[] buffer, int offset)
        {
            return (UInt16)((offset + 1) < buffer.Length ?
                    (UInt16)((UInt16)(buffer[offset + 1] << 8)) | (UInt16)buffer[offset] :
                    0);
        }

        /// <summary>
        /// Gets 4 bytes with offset check
        /// </summary>
        /// <param name="buffer"></param>
        /// <param name="offset"></param>
        /// <returns></returns>
        public static UInt32 Get4Bytes(this byte[] buffer, int offset)
        {
            return (UInt32)((offset + 3) < buffer.Length ?
                   (UInt32)(buffer[offset + 3] << 24) | (UInt32)(buffer[offset + 2] << 16) | (UInt32)(buffer[offset + 1] << 8) | (UInt32)buffer[offset] :
                   0);
        }

        /// <summary>
        /// Gets 4 bytes with offset check
        /// </summary>
        /// <param name="buffer"></param>
        /// <param name="offset"></param>
        /// <returns></returns>
        public static Int32 GetInt32(this byte[] buffer, int offset)
        {
            return (Int32)((offset + 3) < buffer.Length ?
                   (Int32)(buffer[offset + 3] << 24) | (Int32)(buffer[offset + 2] << 16) | (Int32)(buffer[offset + 1] << 8) | (Int32)buffer[offset] :
                   0);
        }

        /// <summary>
        /// Calculates modbus CRC from specified buffer
        /// </summary>
        /// <param name="buf"></param>
        /// <param name="offset"></param>
        /// <param name="length"></param>
        /// <returns></returns>
        public static UInt16 CalcCrc(this byte[] buf, int offset, int length)
        {
            UInt16 crc = 0xFFFF;

            for (int pos = offset; pos < length; pos++)
            {
                crc ^= (UInt16)buf[pos];                // XOR byte into least sig. byte of crc

                for (int i = 8; i != 0; i--)
                {    // Loop over each bit
                    if ((crc & 0x0001) != 0)
                    {      // If the LSB is set
                        crc >>= 1;                      // Shift right and XOR 0xA001
                        crc ^= 0xA001;
                    }
                    else                                // Else LSB is not set
                        crc >>= 1;                      // Just shift right
                }
            }
            
            return crc;
        }
    }
}
