﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DataLayer.Models
{
    public class Packet
    {
        #region Public Properties

        public byte PacketType { get { return Get1Byte(1); } }
        public UInt16 PacketCode { get { return Get2Bytes(2); } }
        public UInt16 CRC { get { return Get2Bytes(PayloadLengt + 5); } }
        public byte PayloadLengt { get { return Get1Byte(4); } }

        #endregion

        #region Public methods

        /// <summary>
        /// Creates specific packet from base packet
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <returns></returns>
        public T CastPacket<T>() where T: Packet
        {
            return Activator.CreateInstance(typeof(T), new object[] { _buffer }) as T;
        }

        /// <summary>
        /// Checks whether CRC from internal buffer is valid (calculated vs received)
        /// </summary>
        /// <returns></returns>
        public bool IsCRCValid()
        {
            var calcCrc = _buffer.CalcCrc(0, _buffer.Length - 2);
            return calcCrc == CRC;
        }

        #endregion

        #region Private Members

        protected byte[] _buffer;

        #endregion

        #region Protected Functions

        public Packet(byte[] buffer)
        {
            _buffer = buffer;
        }

        protected byte Get1Byte(int offset) => _buffer.Get1Byte(offset);
        protected UInt16 Get2Bytes(int offset) => _buffer.Get2Bytes(offset);
        protected UInt32 Get4Bytes(int offset) => _buffer.Get4Bytes(offset);
        protected Int32 GetInt32(int offset) => _buffer.GetInt32(offset);

        #endregion
    }
}
