﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DataLayer.Models
{
    /// <summary>
    /// Packet with mm resolution coordinates
    /// </summary>
    public class Packet47_12 : Packet
    {
        #region Public Properties

        /// <summary>
        /// Number of beacons in packet
        /// </summary>
        public byte BeaconsCnt { get { return Get1Byte(5); } }

        /// <summary>
        /// Beacons data 
        /// </summary>
        public BeaconCoordinates[] Beacons { get { return _beacons.ToArray(); } }

        #endregion

        #region Constructors

        public Packet47_12(byte[] buffer) : base(buffer)
        {
            // initialize beacon data (if available)
            if (this.BeaconsCnt > 0)
            {
                var dataOffset = 6;
                var beaconDataLen = 14;
                for (int i = 0; i < this.BeaconsCnt; ++i)
                {
                    var beaconBuff = new byte[beaconDataLen];
                    Array.Copy(_buffer, dataOffset + beaconDataLen * i, beaconBuff, 0, beaconDataLen);
                    _beacons.Add(new BeaconCoordinates(beaconBuff));
                }
            }
        }

        #endregion

        #region Private Members

        List<BeaconCoordinates> _beacons = new List<BeaconCoordinates>();   // list of beacons information parsed from the buffer

        #endregion
    }
}
