﻿using DataLayer.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DataLayer
{
    /// <summary>
    /// Facade to access data from marvelmind modem
    /// </summary>
    public class ModemFacade
    {
        #region Public Members

        public delegate void PacketReceivedHandler(Packet packet);
        public event PacketReceivedHandler OnPacketReceive;

        #endregion

        #region Public Methods

        /// <summary>
        /// Connects to modem - stars communication with the modem
        /// </summary>
        public void Connect()
        {
            _dataLayer.Start();
        }

        /// <summary>
        /// Disconnects from modem
        /// </summary>
        public void Disconnect()
        {
            _dataLayer.Stop();
        }

        #endregion

        public ModemFacade(IDataLayer dataLayer)
        {
            _dataLayer = dataLayer;
            _dataLayer.OnPacketReceived += DataLayer_PacketReceived;
        }

        #region Private Members

        private IDataLayer _dataLayer = null;

        #endregion

        #region Private Events

        /// <summary>
        /// Event on received data packet from communication layer
        /// </summary>
        /// <param name="buffer"></param>
        private void DataLayer_PacketReceived(byte[] buffer)
        {
            try
            {
                var packet = new Packet(buffer);
                if (packet.IsCRCValid())
                    OnPacketReceive?.Invoke(packet);
            }
            catch { }
        }

        #endregion
    }
}
